@@ -10,6 +10,7 @@ pub struct Account {
     position: Position,
     acc_tracker: AccTracker,
     active_limit_orders: HashMap<u64, Order>,
+    lookup_id_from_user_order_id: HashMap<u64, u64>,
     executed_orders: Vec<Order>,
     // used for calculating hedged order size for order margin calculation
     open_limit_buy_size: f64,
@@ -17,6 +18,7 @@ pub struct Account {
     // TODO: remove following two fields
     min_limit_buy_price: f64,
     max_limit_sell_price: f64,
+    starting_balance: f64,
 }
 
 impl Account {
@@ -30,11 +32,13 @@ impl Account {
             position,
             acc_tracker,
             active_limit_orders: HashMap::new(),
+            lookup_id_from_user_order_id: HashMap::new(),
             executed_orders: vec![],
             open_limit_buy_size: 0.0,
             open_limit_sell_size: 0.0,
             min_limit_buy_price: 0.0,
             max_limit_sell_price: 0.0,
+            starting_balance,
         }
     }
 
@@ -77,6 +81,7 @@ impl Account {
 
     /// Return recently executed orders
     /// and clear them afterwards
+    #[inline(always)]
     pub(crate) fn executed_orders(&mut self) -> Vec<Order> {
         let exec_orders = self.executed_orders.clone();
         self.executed_orders.clear();
@@ -99,8 +104,15 @@ impl Account {
     /// Cancel an active order
     /// returns Some order if successful with given order_id
     pub fn cancel_order(&mut self, order_id: u64) -> Option<Order> {
+        debug!("cancel_order: {}", order_id);
         let removed_order = match self.active_limit_orders.remove(&order_id) {
-            None => return None,
+            None => {
+                debug!(
+                    "order with id {} not found in active limit orders",
+                    order_id
+                );
+                return None;
+            }
             Some(o) => o,
         };
 
@@ -131,11 +143,22 @@ impl Account {
             }
         }
 
-        None
+        Some(removed_order)
+    }
+
+    /// Cancel an active order based on the user_order_id of an Order
+    pub fn cancel_order_by_user_id(&mut self, user_order_id: u64) -> Option<Order> {
+        let id = match self.lookup_id_from_user_order_id.get(&user_order_id) {
+            None => return None,
+            Some(id) => id,
+        };
+        self.cancel_order(*id)
     }
 
     /// Cancel all active orders
     pub fn cancel_all_orders(&mut self) {
+        debug!("cancel_all_orders");
+
         self.margin.set_order_margin(0.0);
         self.open_limit_buy_size = 0.0;
         self.open_limit_sell_size = 0.0;
@@ -177,11 +200,24 @@ impl Account {
         self.margin.set_order_margin(new_om);
 
         self.acc_tracker.log_limit_order_submission();
-        self.active_limit_orders.insert(order.id(), order);
+        let order_id = order.id();
+        match self.active_limit_orders.insert(order_id, order) {
+            None => {}
+            Some(_) => warn!(
+                "there already was an order with this id in active_limit_orders. \
+            This should not happen as order id should be incrementing"
+            ),
+        };
+        match order.user_order_id() {
+            None => {}
+            Some(user_order_id) => {
+                self.lookup_id_from_user_order_id
+                    .insert(*user_order_id, order_id);
+            }
+        };
     }
 
     /// Remove the assigned order margin for a given order
-    #[inline]
     pub(crate) fn remove_executed_order_from_order_margin_calculation(
         &mut self,
         exec_order: &Order,
@@ -194,6 +230,8 @@ impl Account {
         debug_assert!(self.open_limit_buy_size >= 0.0);
         debug_assert!(self.open_limit_sell_size >= 0.0);
 
+        self.active_limit_orders.remove(&exec_order.id()).unwrap();
+
         // re-calculate min and max price
         self.min_limit_buy_price = if self.active_limit_orders.is_empty() {
             0.0
@@ -213,24 +251,7 @@ impl Account {
                 .map(|(_, o)| o.limit_price().unwrap())
                 .fold(f64::NAN, f64::max)
         };
-
-        self.active_limit_orders.remove(&exec_order.id());
-
         let mut new_om: f64 = 0.0;
-        let mut mock_acc = self.clone();
-        let mut validator = Validator::new(fee_maker, 0.0, self.futures_type);
-        let price = exec_order
-            .limit_price()
-            .expect("Only limit orders should be passed into this method");
-        // rough assumption which may be wrong
-        validator.update(price, price);
-        for (_, o) in self.active_limit_orders.iter() {
-            let order_margin = validator
-                .validate_limit_order(o, &mock_acc)
-                .expect("Already submitter limit orders have to be valid");
-            mock_acc.append_limit_order(*o, order_margin);
-            new_om += order_margin;
-        }
 
         self.margin.set_order_margin(new_om);
     }
@@ -424,6 +445,25 @@ mod tests {
         assert_eq!(account.margin().position_margin(), 0.0);
     }
 
+    #[test]
+    fn account_cancel_order_by_user_id() {
+        if let Err(_) = pretty_env_logger::try_init() {}
+
+        let futures_type = FuturesTypes::Inverse;
+        let mut account = Account::new(1.0, 1.0, futures_type);
+        let mut validator = Validator::new(0.0, 0.0, futures_type);
+        validator.update(100.0, 100.1);
+
+        let mut o = Order::limit(Side::Buy, 100.0, 50.0).unwrap();
+        o.set_user_order_id(1000);
+        let order_margin = validator.validate_limit_order(&o, &account).unwrap();
+        account.append_limit_order(o, order_margin);
+        assert!(!account.active_limit_orders().is_empty());
+
+        account.cancel_order_by_user_id(1000).unwrap();
+        assert!(account.active_limit_orders.is_empty());
+    }
+
     #[test]
     fn account_cancel_all_orders() {
         let futures_type = FuturesTypes::Inverse;
